from dalloriam.datahose import DatahoseClient

from tests.mocks.response import MockResponse

from unittest import mock

import pytest


def test_datahose_client_defines_correct_headers():
    password = 'some_password'
    c = DatahoseClient('some_host', password)

    assert c._headers == {
        'Authorization': password
    }


def test_datahose_push_sends_correct_payload():
    with mock.patch('requests.post', return_value=MockResponse(status_code=200)) as mock_post:
        c = DatahoseClient('some_host', 'some_password')
        c.push('some.key', {'some': 'data'})

        mock_post.assert_called_once_with('some_host', json={'key': 'some.key', 'body': {'some': 'data'}}, headers={
            'Authorization': 'some_password'
        })


def test_datahose_push_adds_time_to_payload_when_required():
    with mock.patch('requests.post', return_value=MockResponse(status_code=200)) as mock_post:
        c = DatahoseClient('some_host', 'some_password')
        c.push('some.key', {'some': 'data'}, time=42)

        mock_post.assert_called_once_with(
            'some_host',
            json={
                'key': 'some.key',
                'body': {'some': 'data'},
                'time': 42
            },
            headers={
                'Authorization': 'some_password'
            }
        )


def test_datahose_push_raises_valueerror_when_status_code_isnt_200():
    with mock.patch('requests.post', return_value=MockResponse(status_code=400, text='Something terrible happened')):
        c = DatahoseClient('some_host', 'some_password')

        with pytest.raises(ValueError):
            c.push('some.key', {'some': 'data'})


def test_datahose_notify_preformats_body_for_push():
    c = DatahoseClient('some_host', 'some_password')

    with mock.patch.object(c, 'push') as mock_push:
        c.notify('some_sender', 'some_message')

        mock_push.assert_called_once_with('notification', data={
            'sender': 'some_sender',
            'message': 'some_message'
        })



from dalloriam.filesystem import path

from unittest import mock


def test_location_context_manager():
    mock_dir = 'hello'
    with mock.patch.object(path.os, 'getcwd', return_value=mock_dir), \
            mock.patch.object(path.os, 'chdir') as mock_chdir:

        with path.location('some_path'):
            mock_chdir.assert_called_with('some_path')

        mock_chdir.assert_called_with(mock_dir)



import json
import requests
import os


DATAHOSE_CONFIG_PATH = os.path.expanduser('~/.config/dalloriam/datahose.json')


class DatahoseClient:

    """
    The DatahoseClient allows for easy interfacing with the cloud datahose.
    """

    def __init__(self, service_host: str = None, password: str = None) -> None:
        """
        Constructor
        Args:
            service_host (Optional[str]): Datahose endpoint
            password (Optional[str]): Datahose password.
        """
        if not (service_host and password):
            cfg_dict = self._try_get_disk_config()

            if not service_host:
                if 'service_host' not in cfg_dict:
                    raise ValueError(
                        f'Missing service_host, either in params or in config file ([{DATAHOSE_CONFIG_PATH}]).'
                    )

        self._push_url = service_host

        self._headers = {
            'Authorization': password
        }

    @staticmethod
    def _try_get_disk_config() -> dict:
        if os.path.isfile(DATAHOSE_CONFIG_PATH):
            with open(DATAHOSE_CONFIG_PATH, 'r') as infile:
                return json.load(infile)
        return {}

    def push(self, key: str, data: dict, time: float = None) -> None:
        """
        Push an event to the datahose.
        Args:
            key (str): The event key.
            data (dict): The event data.
            time (float): The time at which the event occured.
        """
        data = {
            'key': key,
            'body': data
        }
        if time is not None:
            data['time'] = time

        resp = requests.post(self._push_url, json=data, headers=self._headers)

        if resp.status_code != 200:
            raise ValueError(resp.text)

    def notify(self, sender: str, message: str) -> None:
        """
        Send a notification via the datahose.
        Args:
            sender (str): Sender of the notification.
            message (str): Markdown-formatted message to send.
        """
        data = {
            'sender': sender,
            'message': message
        }
        self.push('notification', data=data)



from setuptools import setup, find_packages
import re

version = ''
with open('dalloriam/__init__.py') as f:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Version is not set')

readme = 'See https://github.com/dalloriam/python-stdlib for README.'

setup(
    name='dalloriam',
    author='William Dussault',
    author_email='dalloriam@gmail.com',
    url='https://github.com/dalloriam/python-stdlib',
    version=version,
    packages=find_packages(),
    license='MIT',
    description='Personal standard library.',
    long_description=readme,
    install_requires=['requests']
)


from contextlib import contextmanager

from dalloriam import filesystem
from dalloriam.docker.container import Container

from typing import Dict, Iterator

import os
import random
import string
import sh
import time


class DockerClient:

    def __init__(self, username: str = None, password: str = None, server: str = None) -> None:
        """
        Initializes the client
        Args:
            username (str): Docker repository username.
            password (str): Docker repository password.
            server (str): Docker repository URL.
        """
        self._username = username
        self._password = password
        self._server = server

        if (self._username and self._password) or self._server:
            self._login()

    @staticmethod
    def _format_image_name(image_name: str, tag: str) -> str:
        return f'{image_name}:{tag}'

    def _login(self) -> None:
        args = []

        if self._username:
            args += ['-u', self._username]

        if self._password:
            args += ['-p', self._password]

        if self._server:
            args.append(self._server)

        sh.docker('login', *args)

    def build(self, content_dir: str, image_name: str, tag: str = 'latest') -> None:
        """
        Builds a docker image from a directory.
        Args:
            content_dir (str): Path that contains the image-related files.
            image_name (str): Desired name of the built image.
            tag (str):        Desired tag of the built image.
        """
        with filesystem.location(os.path.abspath(content_dir)):
            sh.docker('build', '-t', self._format_image_name(image_name, tag), '.')

    def push(self, image_name: str, tag: str = 'latest') -> None:
        """
        Pushes a docker image to the target repository
        Args:
            image_name (str): Name of the image to push.
            tag (str): Tag of the image.
        """
        sh.docker('push', self._format_image_name(image_name, tag))

    @contextmanager
    def container(
            self,
            image_name: str,
            tag: str = 'latest',
            ports: Dict[int, int] = None,
            volumes: Dict[str, str] = None) -> Iterator[Container]:
        """
        Start a container with the specified configuration in a context.
        Args:
            image_name (str): Name of the image to start.
            tag (str): Tag of the image to start.
            ports (Dict[int, int]): Port mappings.
            volumes (Dict[str, str]): Volume mappings.

        Returns:
            Running container.
        """
        c = Container(image_name=image_name, tag=tag)
        c.start(ports, volumes)
        time.sleep(2)
        try:
            yield c
        finally:
            c.stop()



from contextlib import contextmanager

import os


@contextmanager
def location(target_path: str) -> None:
    """
    Context manager that facilitates switching to a directory.
    Args:
        target_path: Path to switch to, temporarily.
    """
    old_path = os.getcwd()
    os.chdir(target_path)
    yield
    os.chdir(old_path)
